@@ -10,7 +10,7 @@ import sys
 
 from .contextvar import GVAR
 from .util import timeblock, WaitGroup, nullacontext, optional_cm, make_prependable
-from .resource import ZERO
+from .resource import ZERO, Reject
 from .pair import SocketChannel, SocketChannelAcceptor
 from .subproc import subprocess_pool_daemon, BrokenSubprocessError
 from .stat import log_event, log_timestart, log_timeend, stat_logger, ls_worker_res, reg_probe, collect_probe
@@ -20,6 +20,9 @@ FULL_HEALTH = 3
 STATSPAN = 15 # minutes
 HEARTBEAT_INTERVAL, HEARTBEAT_TOLERANCE = 10, 3 # 10s * 3
 
+pickle_dumps = partial(pickle.dumps, protocol=4)
+pickle_loads = pickle.loads
+
 class GrainRemote(object):
     def __init__(self, addr, res):
         self._c = None
@@ -49,7 +52,7 @@ class GrainRemote(object):
         async with self._c:
             task_status.started()
             async for x in heartbeat_n_receive(self._c):
-                tid, r = pickle.loads(x)
+                tid, r = pickle_loads(x)
                 rq = self.resultq.get(abs(tid))
                 if rq:
                     await rq.send((tid>0, r))
@@ -68,7 +71,7 @@ class GrainRemote(object):
         with self.wg:
             self.resultq[tid], rq = trio.open_memory_channel(0)
             try:
-                await self._c.send(pickle.dumps((tid, res, fn)))
+                await self._c.send(pickle_dumps((tid, res, fn)))
                 with optional_cm(trio.fail_after, getattr(res,'T',-180)+180): # 3min grace period
                     ok, r = await rq.receive()
             except (trio.ClosedResourceError, trio.EndOfChannel):
@@ -109,10 +112,10 @@ class _cobj:
         if GVAR.instance == "local":
             return getattr(self._h, attr)
         return getattr(self._dh, attr)
-class GrainPseudoRemote(object):
+class GrainPseudoRemote:
     def __init__(self, res, out=""):
         self.res = res
-        self.name = "local"
+        self.name = f"{trio.socket.gethostname()}(local)"
         self.health = FULL_HEALTH
         self.wg = WaitGroup()
         self.cg = set()
@@ -148,7 +151,7 @@ class GrainPseudoRemote(object):
         print(f"worker {self.name} clear")
 
 
-class GrainManager(object):
+class GrainManager:
     """Manage workers and resources.
     """
     def __init__(self, pool_init, _n, temperr, persistent, listen_addr):
@@ -173,7 +176,7 @@ class GrainManager(object):
             if w.health <= 0 and w in self.pool:
                 if type(err) not in self.temperr:
                     print(tb)
-                if w.name == 'local': raise RuntimeError("local worker quits")
+                if w.name.endswith('(local)'): raise RuntimeError("local worker quits")
                 print(f"quit worker {w.name} due to poor health {w.health}")
                 await self.unregister(w.name, locked=True)
 
@@ -191,12 +194,21 @@ class GrainManager(object):
     async def unregister(self, name, locked=False):
         async with nullacontext() if locked else self.cond_res:
             # expect one and only name
-            i,w = next(((i,x) for i,x in enumerate(self.pool) if x.name==name), (0,0))
+            i,w = next(((i,x) for i,x in enumerate(self.pool) if x.name==name), (0,None))
             if not i: return
             self.pool.pop(i)
             await w.aclose() # notify exit and reschedule pending jobs
             if not self.persistent:
                 raise RuntimeError(f"worker {name} exits, abort.")
+    async def terminate(self, name):
+        async with self.cond_res:
+            # expect one and only name
+            w = next((x for x in self.pool if x.name==name), None)
+            if not w or type(w.res) is Reject: return
+            w.res = Reject(w.res)
+        await w.wg.wait() # wait till no job is running
+        await self.unregister(name)
+
     async def worker_manager(self, task_status=trio.TASK_STATUS_IGNORED):
         async with trio.open_nursery() as _n, \
                    SocketChannelAcceptor(self.listen_addr, _n=_n) as self.soacceptor:
@@ -210,13 +222,13 @@ class GrainManager(object):
                     continue
                 cmd, msg = msg[:3], msg[3:]
                 if cmd == b"CON": # connect
-                    vaddr, res = pickle.loads(msg)
+                    vaddr, res = pickle_loads(msg)
                     print(f"worker {vaddr} joined with {res}")
                     await self.register(GrainReverseRemote(_c, vaddr, res), _n)
                     continue
                 async with _c: # The following are ephemeral cmds
                     if cmd == b"REG": # register
-                        addr, res = pickle.loads(msg)
+                        addr, res = pickle_loads(msg)
                         print(f"worker {addr} joined with {res}")
                         await self.register(GrainRemote(addr, res), _n)
                     elif cmd == b"UNR": # unregister
@@ -227,6 +239,10 @@ class GrainManager(object):
                         sta = ls_worker_res(self.pool) + '\n' + \
                               collect_probe()
                         await _c.try_send(sta.encode())
+                    elif cmd == b"TRM": # graceful termination
+                        addr = msg.decode()
+                        print(f"worker {addr} is going to leave")
+                        _n.start_soon(self.terminate, addr)
                     else:
                         print(f"worker manager received unknown command {cmd} from {_c.host}")
 
@@ -280,10 +296,14 @@ class GrainExecutor(object):
 
     async def asubmit(self, res, fn, *args, **kwargs):
         self._wg.add()
-        await self.push_job.send((0, res, partial(fn, *args, **kwargs)))
+        tid = self.jobn = self.jobn+1
+        await self.push_job.send((tid, res, partial(fn, *args, **kwargs)))
+        return tid
     def submit(self, res, fn, *args, **kwargs):
         self._wg.add()
-        self.push_job.send_nowait((0, res, partial(fn, *args, **kwargs)))
+        tid = self.jobn = self.jobn+1
+        self.push_job.send_nowait((tid, res, partial(fn, *args, **kwargs)))
+        return tid
     def resubmit(self, tid, res, fn):
         self._wg.add()
         self.push_job.cutin_nowait((tid, res, fn)) # prepend the queue
@@ -323,7 +343,6 @@ class GrainExecutor(object):
                        trio.open_nursery() as _n, \
                        self.pull_job:
                 async for tid, res, fn in self.pull_job:
-                    if not tid: tid = self.jobn = self.jobn+1
                     res, w = await self.mgr.schedule(res)
                     _n.start_soon(self.__task_with_res, tid, res, w, fn)
 
